#include "Five State Button.h"

// Constant Five States for Button  
enum FS_State
{
	FS_NORMAL = 1,
	FS_OVER,
	FS_PRESSED,
	FS_FOCUSED,
	FS_ENABLED
};

// Location for extra window bytes for data 
enum FS_DataOffSet
{
	FSD_STATE	=  0,
	FSD_FOCUSED	=  4,
	FSD_ENABLED =  8,
	FSD_HBITMAP	= 12
};


LRESULT CALLBACK FS_ButtonWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
LONG GetState(HWND hWndFSButton);
void SetState(HWND hWndFSButton,LONG lValue);
void DrawFS_State(LONG iState,HDC hdc,HWND hWndFSButton);
void ForceRedraw(HWND hWnd);

// Force a window to redraw itself(Client Area).
void ForceRedraw(HWND hWnd)
{
	RECT rc;
	GetClientRect(hWnd,&rc);
	InvalidateRect(hWnd,&rc,TRUE);
}

// Create a new FS Button window and set bitmap image to it.
HWND CreateFS_Button (HBITMAP hbmImage,HMENU hMenu,HWND hWndParent,int x,int y,int nWidth,int nHeight,HINSTANCE hInstance)
{
	HWND hWndRet;
	hWndRet = CreateWindow(FSBUTTON, NULL, WS_CHILD | WS_VISIBLE | WS_TABSTOP ,x, y, nWidth,nHeight,hWndParent, hMenu, hInstance, NULL);

	if (hWndRet == NULL)
		return(NULL);

	SetBitmapFS_Button(hbmImage,hWndRet);

	return(hWndRet);
}

// set bitmap picture for button window in extra bytes location.
void SetBitmapFS_Button(HBITMAP hbmImage,HWND hWndFSButton)
{
	SetWindowLong(hWndFSButton,FSD_HBITMAP,(LONG) hbmImage);
}

// Get current state from window extra bytes
LONG GetState(HWND hWndFSButton)
{
	return (GetWindowLong(hWndFSButton,FSD_STATE));
}

// Set current state to window extra bytes
void SetState(HWND hWndFSButton,LONG lValue)
{
	SetWindowLong(hWndFSButton,FSD_STATE,lValue);
}

// Register Five State Button Class 
ATOM InitFS_Button(HINSTANCE hInstance)
{
	WNDCLASSEX wndClsEx;

	ZeroMemory( &wndClsEx , sizeof(wndClsEx));

	wndClsEx.cbSize			= sizeof(wndClsEx);
	wndClsEx.lpfnWndProc	= (WNDPROC)FS_ButtonWndProc;
	wndClsEx.hInstance		= hInstance;
	wndClsEx.hCursor		= LoadCursor(NULL, IDC_ARROW);
	wndClsEx.lpszClassName	= FSBUTTON;
	wndClsEx.style			= CS_HREDRAW | CS_OWNDC | CS_VREDRAW;
	wndClsEx.cbWndExtra		= sizeof(LONG) * 4;// extra bytes for saving state and bitmap handle.

	return RegisterClassEx(&wndClsEx);
}	

// FS Button window procedure 
LRESULT CALLBACK FS_ButtonWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	PAINTSTRUCT ps;
	HDC hdc;
	TRACKMOUSEEVENT tme;
	LONG iState;


	switch (uMsg) 
	{
	case WM_CREATE:
		// set state to enabled
		SetWindowLong(hWnd,FSD_ENABLED,TRUE);
		// set state to normal
		SetState(hWnd,FS_NORMAL);
		break;
	case WM_SETFOCUS:
		// set state focus to true
		SetWindowLong(hWnd,FSD_FOCUSED,TRUE);
		// force to redraw button
		ForceRedraw(hWnd);
		break;
	case WM_KILLFOCUS:
		// set state focus to false
		SetWindowLong(hWnd,FSD_FOCUSED,FALSE);
		// force to redraw button
		ForceRedraw(hWnd);
		break;

	case WM_ENABLE:
		// Set window enable state (depend on wParam)
		SetWindowLong(hWnd,FSD_ENABLED,(LONG) wParam);
		// force to redraw button
		ForceRedraw(hWnd);
		break;

	case  WM_LBUTTONUP:
		iState = GetState(hWnd);
		if (iState != FS_OVER)
		{	
			// Set state to Mouse Over
			SetState(hWnd,FS_OVER);
			// Generate Click event
			SendMessage(GetParent(hWnd),WM_COMMAND,MAKEWPARAM((int)GetWindowLong(hWnd,GWL_ID),BN_CLICKED),(LPARAM)hWnd);
			// force to redraw button
			ForceRedraw(hWnd);

		}
		break;

	case  WM_LBUTTONDOWN:
		iState = GetState(hWnd);
		if (iState != FS_PRESSED)
		{
			// Set state to pressed
			SetState(hWnd,FS_PRESSED);
			// force to redraw button
			ForceRedraw(hWnd);
		}
		break;

	case WM_MOUSEMOVE:
		iState = GetState(hWnd);
		if (iState != FS_OVER && iState != FS_PRESSED)
		{

			// Set state to Mouse Over
			SetState(hWnd,FS_OVER);

			tme.cbSize =  sizeof(tme);
			tme.dwFlags = TME_LEAVE;
			tme.hwndTrack = hWnd;
			tme.dwHoverTime = 0 ;
			
			// track mouse for leave event
			_TrackMouseEvent(&tme);
			// force to redraw button
			ForceRedraw(hWnd);
		}
		break;

	case WM_MOUSELEAVE:
		iState = GetState(hWnd);
		if (iState != FS_NORMAL)
		{
			// Set state to Normal
			SetState(hWnd,FS_NORMAL);//iState = FS_NORMAL;
			// force to redraw button
			ForceRedraw(hWnd);
		}
		break;

	case WM_ERASEBKGND:
		return (LRESULT)1;

	case WM_PAINT:// Draw Button Control Window depending on state	
		iState = GetState(hWnd);
		hdc = BeginPaint(hWnd, &ps);

		if (GetWindowLong(hWnd,FSD_ENABLED) == TRUE)
		{
			if (iState == FS_OVER)
				DrawFS_State(FS_OVER,hdc,hWnd);

			if (iState ==  FS_PRESSED)
				DrawFS_State(FS_PRESSED,hdc,hWnd);

			if (iState == FS_NORMAL)
			{
				if (GetWindowLong(hWnd,FSD_FOCUSED) ==  TRUE)
					DrawFS_State(FS_FOCUSED,hdc,hWnd);
				else
					DrawFS_State(FS_NORMAL,hdc,hWnd);
			}
		}
		else
		{
			DrawFS_State(FS_ENABLED,hdc,hWnd);
		}
		EndPaint(hWnd, &ps);
		break;
	}
	return DefWindowProc(hWnd, uMsg, wParam, lParam);
}

// Draw button state
void DrawFS_State(LONG iState,HDC hdc,HWND hWndFSButton)
{

	// Check for valid  Bitmap handle.
	if ( GetWindowLong(hWndFSButton,FSD_HBITMAP)  == 0 )
		return;

	//...
	HDC hdcImage;
	HGDIOBJ oldobj;

	// Create a device context (DC) in memory and select Bitmap in it. 
	hdcImage = CreateCompatibleDC( hdc );
	oldobj = SelectObject( hdcImage, (HGDIOBJ) GetWindowLong(hWndFSButton,FSD_HBITMAP) );

	//...
	RECT rc;
	HDC hdcMem;
	HBITMAP hbmMem, hbmOld;

	GetClientRect(hWndFSButton, &rc);
	// Create a memory DC and Bitmap for buffer drawing. 
	hdcMem = CreateCompatibleDC( hdc );
	hbmMem = CreateCompatibleBitmap(hdc,rc.right,rc.bottom);
	hbmOld = (HBITMAP) SelectObject(hdcMem, hbmMem);

	//...
	LONG lXSrc;

	lXSrc = iState * rc.right;
	lXSrc -= rc.right;

	// Draw to memory device context 
	BitBlt(hdcMem,0, 0,rc.right, rc.bottom,hdcImage,lXSrc, 0,SRCCOPY);
	// Draw state to control
	BitBlt(hdc,0, 0,rc.right, rc.bottom,hdcMem,0, 0,SRCCOPY);

	//...
	SelectObject(hdcMem, hbmOld);
	DeleteObject(hbmMem);
	DeleteDC(hdcMem);

	SelectObject( hdcImage,oldobj );
	DeleteDC( hdcImage);
}
/*
This software created by Muhammad Arshad Latti
From Sargodha ( Pakistan ) 
e-mail   :: arshadlatti@gmail.com 
web_site :: http://www.geocities.com/arshadlati
*/